{% extends 'bom/base.html' %}

{% load materializecss %}

{% block title %}Sign Up For BOM{% endblock %}
{% block head-title %}Sign Up{% endblock %}

{% block content %}
    {% load static %}
    <div class="container">
        <div class="section center">
            <h5 class="">Sign up for BOM</h5>
        </div>
        <div class="section center">
            <form method="post">
                {% csrf_token %}
                <div class="row">
                    <div class="col m6 push-m3 s12">
                        {{ form|materializecss }}
                        <div class="center">
                            <button class="waves-effect waves-light btn green lighten-1" style="min-width:150px" type="submit">Sign Up</button>
                        </div>
                    </div>
                </div>
            </form>
        </div>
    </div>
{% endblock %}